#!/usr/bin/env python
"""Run a search against DXR.

Usage: dxr [options] <query>...

Options:
  --case-insensitive  Perform a case-insensitive search (searches are
                      case-sensitive by default).
  --grep              Grep-style "FILE:LINE" output
  -h --help           Show this screen.
  --limit=LIMIT       Maximum number of matches [default: 50]
  --no-highlight      Disable syntax highlighting.
  --pager=PROGRAM     Direct output through PROGRAM.
  --server=DOMAIN     DXR instance to send the search request to.
                      [default: https://dxr.mozilla.org]
  --style=STYLE       Name of Pygments style for syntax highlighting.
                      [default: paraiso-dark]
  --tree=TREE         Code tree to search against.
                      [default: mozilla-central]
  -v --version        Show program version.
"""
import re
import subprocess
from HTMLParser import HTMLParser
from math import ceil, log10

import requests
from blessings import Terminal
from docopt import docopt
from pygments import highlight
from pygments.lexers import get_lexer_for_filename
from pygments.formatters import Terminal256Formatter
from pygments.util import ClassNotFound


__version__ = '0.2.1'


def main():
    arguments = docopt(__doc__, version=__version__)
    html_parser = HTMLParser()
    t = Terminal()
    output = []

    try:
        limit = int(arguments['--limit'])
    except:
        limit = 50  # Sigh. Numbers are so hard to type.

    url = '{server}/{tree}/search'.format(server=arguments['--server'],
                                          tree=arguments['--tree'])
    response = requests.get(url, params={
        'q': ' '.join(arguments['<query>']),
        'redirect': False,
        'format': 'json',
        'case': not arguments['--case-insensitive'],
        'limit': limit,
        'offset': 0
    })

    if response.status_code != 200:
        output.append('Search failed: DXR returned {status} {reason}'.format(
            status=response.status_code,
            reason=response.reason,
        ))
    else:
        grep_style = arguments['--grep']
        response_json = response.json()
        formatter = Terminal256Formatter(style=arguments['--style'])

        if len(response_json['results']) < 1:
            output.append('No results found.')
        else:
            for result in response_json['results']:
                if not grep_style:
                    output.append(t.green(result['path']))

                # Find maximum line number length so we can line them up.
                max_log = max(log10(l['line_number']) for l in result['lines'])
                lineno_len = int(ceil(max_log))
                if grep_style:
                    line_template = (u'{t.green}{path}:{t.yellow}{{lineno}}{t.normal}:{{line}}'
                                     .format(t=t, path=result['path']))
                else:
                    line_template = (u'  {t.yellow}{{lineno:>{lineno_len}}}{t.normal} {{line}}'
                                     .format(t=t, lineno_len=lineno_len))

                try:
                    lexer = get_lexer_for_filename(result['path'])
                except ClassNotFound:
                    lexer = None  # No highlighting for you!

                for line_result in result['lines']:
                    line = html_parser.unescape(line_result['line'])

                    # Find highlight term and remove <b> tags.
                    match = re.search(r'<b>(?P<term>.+)</b>', line)
                    highlight_term = match.group('term') if match else None
                    line = line.replace('<b>', '').replace('</b>', '')

                    # Highlight normally and add underline.
                    if not arguments['--no-highlight'] and lexer:
                        line = highlight(line, lexer, formatter).strip('\n')
                    line = line.replace(highlight_term, t.underline_bold(highlight_term))

                    output.append(line_template.format(
                        lineno=unicode(line_result['line_number']),
                        line=line
                    ))
                if not grep_style:
                    output.append('')

    output = '\n'.join(output)
    if arguments['--pager']:
        p = subprocess.Popen(arguments['--pager'], stdin=subprocess.PIPE, shell=True)
        p.communicate(input=output)
        p.wait()
    else:
        print output


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import codecs
import os
import re
from setuptools import setup


def read(*parts):
    return codecs.open(os.path.join(os.path.dirname(__file__), *parts), encoding='utf8').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='dxr-cmd',
    description='Command-line tool for querying DXR.',
    long_description=read('README.rst'),
    version=find_version('dxr.py'),
    py_modules=['dxr'],
    author='Michael Kelly',
    author_email='me@mkelly.me',
    url='https://github.com/Osmose/dxr',
    license='MIT',
    install_requires=[
        'blessings>=1.6',
        'requests>=2.5.1',
        'docopt>=0.6.2',
        'pygments>=2.0.2',
        'pyopenssl>=0.13',
        'ndg-httpsclient>=0.3.2',
        'pyasn1>=0.1.6'
    ],
    include_package_data=True,
    entry_points={
      'console_scripts': [
          'dxr = dxr:main'
      ]
   }
)
